//+------------------------------------------------------------------+
//|                                                 StalinSignal.mqh |
//|                             Copyright  2011,   Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link      "farria@mail.redcom.ru"
//+------------------------------------------------------------------+
//|                                                   |
//+------------------------------------------------------------------+
#property tester_indicator "Stalin.ex5"
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+ 
//|                                                |
//+------------------------------------------------------------------+ 
#define OPEN_LONG     80  //          
#define OPEN_SHORT    80  //          
#define CLOSE_LONG    40  //          
#define CLOSE_SHORT   40  //          
#define REVERSE_LONG  100 //         
#define REVERSE_SHORT 100 //         
#define NO_SIGNAL      0  //         
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=,    Stalin                   |
//| Type=SignalAdvanced                                              |
//| Name=Stalin                                                      |
//| Class=CStalinSignal                                              |
//| Page=                                                            |
//| Parameter=BuyPosOpen,bool,true,           |
//| Parameter=SellPosOpen,bool,true,          |
//| Parameter=BuyPosClose,bool,true,      |
//| Parameter=SellPosClose,bool,true,     |
//| Parameter=Ind_Timeframe,ENUM_TIMEFRAMES,PERIOD_H4,      |
//| Parameter=MAMethod,ENUM_MA_METHOD,MODE_EMA,       |
//| Parameter=MAShift,uint,0,                      |
//| Parameter=Fast,uint,14,                      |
//| Parameter=Slow,uint,21,                    |
//| Parameter=RSI,uint,17, RSI                                 |
//| Parameter=Confirm,uint,0,                         |
//| Parameter=Flat,uint,0,               |
//| Parameter=SignalBar,uint,1,              |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//|  CStalinSignal.                                             |
//| :            |
//|  Stalin http://www.mql5.com/ru/code/487/.              |
//|     CExpertSignal.                    |
//+------------------------------------------------------------------+
class CStalinSignal : public CExpertSignal
  {
protected:
   CiCustom          m_indicator;        //      Stalin

   //---  
   bool              m_BuyPosOpen;       //     
   bool              m_SellPosOpen;      //     
   bool              m_BuyPosClose;      //     
   bool              m_SellPosClose;     //     
   ENUM_TIMEFRAMES   m_Ind_Timeframe;    //   Stalin
   ENUM_MA_METHOD    m_MAMethod;         //  
   uint              m_MAShift;          //      
   uint              m_Fast;             //   
   uint              m_Slow;             //   
   uint              m_RSI;              //  RSI
   uint              m_Confirm;          //   
   uint              m_Flat;             //      
   uint              m_SignalBar;        //     

public:
                     CStalinSignal();

   //---   
   void               BuyPosOpen(bool value)                  { m_BuyPosOpen=value;       }
   void               SellPosOpen(bool value)                 { m_SellPosOpen=value;      }
   void               BuyPosClose(bool value)                 { m_BuyPosClose=value;      }
   void               SellPosClose(bool value)                { m_SellPosClose=value;     }
   void               Ind_Timeframe(ENUM_TIMEFRAMES value)    { m_Ind_Timeframe=value;    }
   void               MAMethod(ENUM_MA_METHOD value)          { m_MAMethod=value;         }
   void               MAShift(uint value)                     { m_MAShift=value;          }
   void               Fast(uint value)                        { m_Fast=value;             }
   void               Slow(uint value)                        { m_Slow=value;             }
   void               RSI(uint value)                         { m_RSI=value;              }
   void               Confirm(uint value)                     { m_Confirm=value;          }
   void               Flat(uint value)                        { m_Flat=value;             }
   void               SignalBar(uint value)                   { m_SignalBar=value;        }

   //---    
   virtual bool      ValidationSettings();
   //---    
   virtual bool      InitIndicators(CIndicators *indicators); //  
   //---      
   virtual int       LongCondition();
   virtual int       ShortCondition();

   //--- ,   Stalin
   bool              InitStalin(CIndicators *indicators);
  };
//+------------------------------------------------------------------+
//|  CStalinSignal.                                       |
//| INPUT:  .                                                     |
//| OUTPUT: .                                                     |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
void CStalinSignal::CStalinSignal()
  {
//---    
   m_BuyPosOpen=true;
   m_SellPosOpen=true;
   m_BuyPosClose=true;
   m_SellPosClose=true;

//---   
   m_Ind_Timeframe=PERIOD_H4;
   m_MAShift=0;
   m_MAMethod=MODE_EMA;
   m_Fast=14;
   m_Slow=21;
   m_RSI=17;
   m_Confirm=0;
   m_Flat=0;
//---  
   m_SignalBar=1;
   m_used_series=USE_SERIES_OPEN+USE_SERIES_HIGH+USE_SERIES_LOW+USE_SERIES_CLOSE;
  }
//+------------------------------------------------------------------+
//|   .                                   |
//| INPUT:  .                                                     |
//| OUTPUT: true-  ,  false.             |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CStalinSignal::ValidationSettings()
  {
//---  
   if(m_Fast==0)
     {
      printf(__FUNCTION__+":        ");
      return(false);
     }

   if(m_Slow==0)
     {
      printf(__FUNCTION__+":        ");
      return(false);
     }

   if(m_Slow==0)
     {
      printf(__FUNCTION__+":  RSI     ");
      return(false);
     }
//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|    .                           |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CStalinSignal::InitIndicators(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---  
   if(!InitStalin(indicators)) return(false);

//---  
   return(true);
  }
//+------------------------------------------------------------------+
//|   Stalin.                                 |
//| INPUT:  indicators -   -               |
//|                        .                    |
//| OUTPUT: true-   ,  false.         |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
bool CStalinSignal::InitStalin(CIndicators *indicators)
  {
//---  
   if(indicators==NULL) return(false);

//---    
   if(!indicators.Add(GetPointer(m_indicator)))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---   
   MqlParam parameters[10];

   parameters[0].type=TYPE_STRING;
   parameters[0].string_value="Stalin.ex5";

   parameters[1].type=TYPE_UINT;
   parameters[1].integer_value=MODE_EMA;

   parameters[2].type=TYPE_UINT;
   parameters[2].integer_value=m_MAShift;

   parameters[3].type=TYPE_UINT;
   parameters[3].integer_value=m_Fast;

   parameters[4].type=TYPE_UINT;
   parameters[4].integer_value=m_Slow;

   parameters[5].type=TYPE_UINT;
   parameters[5].integer_value=m_RSI;

   parameters[6].type=TYPE_UINT;
   parameters[6].integer_value=m_Confirm;

   parameters[7].type=TYPE_UINT;
   parameters[7].integer_value=m_Flat;

   parameters[8].type=TYPE_UINT;
   parameters[8].integer_value=0;

   parameters[9].type=TYPE_UINT;
   parameters[9].integer_value=0;

//---     
   if(!m_indicator.Create(m_symbol.Name(),m_Ind_Timeframe,IND_CUSTOM,10,parameters))
     {
      printf(__FUNCTION__+":   ");
      return(false);
     }

//---  
   if(!m_indicator.NumBuffers(2)) return(false);

//---     Stalin

   return(true);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CStalinSignal::LongCondition()
  {
//---       1  Stalin
   double Signal=m_indicator.GetData(1,m_SignalBar);

//---    
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_BuyPosOpen)
        {
         if(m_SellPosClose) return(REVERSE_SHORT);
         else return(OPEN_LONG);
        }
      else
        {
         if(m_SellPosClose) return(CLOSE_SHORT);
        }
     }

//---      
   if(!m_SellPosClose) return(NO_SIGNAL);

   int Bars_=Bars(m_symbol.Name(),m_Ind_Timeframe);

   for(int bar=int(m_SignalBar); bar<Bars_; bar++)
     {
      Signal=m_indicator.GetData(0,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(NO_SIGNAL);

      Signal=m_indicator.GetData(1,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_SHORT);
     }

//---   
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
//|                              |
//|                            |
//| INPUT:                                                        |
//| OUTPUT:    0  100                                   |
//| REMARK: .                                                     |
//+------------------------------------------------------------------+
int CStalinSignal::ShortCondition()
  {
//---       0  Stalin
   double Signal=m_indicator.GetData(0,m_SignalBar);

//---   
   if(Signal && Signal!=EMPTY_VALUE)
     {
      if(m_SellPosOpen)
        {
         if(m_BuyPosClose) return(REVERSE_LONG);
         else return(OPEN_SHORT);
        }
      else
        {
         if(m_BuyPosClose) return(CLOSE_LONG);
        }
     }

//---      
   if(!m_BuyPosClose) return(NO_SIGNAL);

   int Bars_=Bars(Symbol(),m_Ind_Timeframe);
   for(int bar=int(m_SignalBar); bar<Bars_; bar++)
     {
      Signal=m_indicator.GetData(1,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(NO_SIGNAL);

      Signal=m_indicator.GetData(0,bar);
      if(Signal && Signal!=EMPTY_VALUE) return(CLOSE_LONG);
     }

//---      
   return(NO_SIGNAL);
  }
//+------------------------------------------------------------------+
